//JWH / NOVEMBER 2000
//
//  TEST APPLICATION FOR IOBJECT.DLL
//  REMEMBER: IOJECT IS *NAMEWARE*
//
//    Here after you will find 3 ways to create an instance of the IObject.dll component.
//    First & second, with the traditionnal CLSID and IID declarations
//    Third, using the extended registry settings of iobject.dll to retrieve IDs

//    For this code to run: create the iobject.dll and self-register it: regsvr32 iobject.dll
//      Then create a new lcc-win32 project: => Console app, add this file and libraries: ole32.lib, uuid.lib
//      Make and enjoy the power of COM (i'm kidding :)
//      After execution check the co_debug.txt file to see the COM calls
//
//    Note: Usualy macros will mask the ->lpVtbl-> indirection. However
//          for this small example all the mechanics is apparent.
//
#include <windows.h>
#include <objbase.h>
#include <initguid.h>
#include <stdio.h>

//Object and GUIDs definitions
#define IID_BOTH_MODE  //In this demo  we use both, static and dynamic GUIDs
                       //so we need IDs definitions. But a client using solely
					   //dynamic mode won't define this and so wont get
					   //static GUIDs defines
#include "iobject.h"

int main(int argc, char **argv)
{
IInterf1 *pif1;
IInterf2 *pif2;
IClassFactory *pCF;
HRESULT hr;
int result;


  //DON'T FORGET
  CoInitialize(NULL);

//FIRST, the simplest way using static IDs and ignoring classFactory:

    printf("1- CoCreateInstance\n");
	hr=CoCreateInstance(&CLSID_IObject, NULL, CLSCTX_INPROC_SERVER, &IID_Interf1, &pif1);

    if(SUCCEEDED(hr)) {
	  char ver[16];
	  pif1->lpVtbl->GetVersion(pif1,ver);
	  pif1->lpVtbl->SetValue(pif1,11);
	  pif1->lpVtbl->GetValue(pif1,&result);
	  pif1->lpVtbl->Release(pif1);
      printf("#%s\n",ver);
	  printf("CREATED IObject, value: %i (should be 11)\n",result);
    } else printf("**FAILED To Create IObject\n");


//SECOND, the fastest way if multiple objects are needed. Use static IDs
//  and shows how to access other interface:

    printf("2- CoGetClassObject\n");
	hr=CoGetClassObject(&CLSID_IObject, CLSCTX_INPROC_SERVER, NULL, &IID_IClassFactory, &pCF);

    if(SUCCEEDED(hr)) {
	  pCF->lpVtbl->LockServer(pCF,TRUE);   //MANDATORY (COM RULE)
	  hr = pCF->lpVtbl->CreateInstance(pCF, NULL, &IID_Interf1, &pif1);  //Create object

      if(SUCCEEDED(hr)) {
	    pif1->lpVtbl->SetValue(pif1,42);
	    pif1->lpVtbl->GetValue(pif1,&result);
		printf("CREATED  IObject, value: %04i (should be 42)\n",result);
		hr=pif1->lpVtbl->QueryInterface(pif1,&IID_Interf2,&pif2); //Get to other interface

		if(SUCCEEDED(hr)) {
	      pif2->lpVtbl->SquareValue(pif2);
	      pif2->lpVtbl->Release(pif2);   //Release interface
	      pif1->lpVtbl->GetValue(pif1,&result);
		  printf("MODIFIED IObject, value: %i (should be 42x42)\n",result);
        } else printf("**FAILED To Obtain IObject Second Interface\n");

	    pif1->lpVtbl->Release(pif1); //Release interface=>delete object
      } else printf("**FAILED To Obtain IObject First Interface\n");

	  printf("LOOP\n");
	  for(int i=0; i<11; i++) {
	    hr = pCF->lpVtbl->CreateInstance(pCF, NULL, &IID_Interf1, &pif1);

        if(SUCCEEDED(hr)) {
		    pif1->lpVtbl->SetValue(pif1,(i+1)*(i+1));
	        pif1->lpVtbl->GetValue(pif1,&result);
			printf("CREATED Object: #%02i, value: %03i (should be %02ix%02i)\n",i+1,result,i+1,i+1);
	        pif1->lpVtbl->Release(pif1);
		} else printf("**FAILED To Create IObject: #%i\n",i+1);

	  }

	  pCF->lpVtbl->LockServer(pCF,FALSE);
	  pCF->lpVtbl->Release(pCF);

    } else printf("**FAILED To Obtain ClassFactory Interface\n");


//THIRD: the extended way. NO STATIC IDs required (IID_BOTH_MODE not defined and EXTENDED_REG defined).
//  Only programatic names are used. This only works if you register IObject.dll with
//  its extended registry flag on. Also maybe this breaks COM rules because Interface
//  IDs aren't suposed to be at this place in the registry. I don't know. It's up to you,
//  but it's really kool to forget about those nasty IDs don't you think ?
//  (
//    True COM object doesn't expose interfaces like this but via IDispatch a TypeLiBrary
//    (TLB file), or Moniker but this require MIDL, lot of time and code etc etc...
//  )
//  Also note that this method can be *time consuming*. Because as CoCreateInstance
//    it involves searching through the registry

#ifdef EXTENDED_REG
    { CLSID cid;
      IID id1;

	  printf("3- Using CLSIDFromProgID\n");
	  hr=CLSIDFromProgID(szPROGID,&cid);  //Get class ID

	  if(SUCCEEDED(hr)) {
	    hr=CLSIDFromProgID(szIIF1,&id1);  //Get Interf1 ID

	    if(SUCCEEDED(hr)) {
	      hr=CoCreateInstance(&cid, NULL, CLSCTX_INPROC_SERVER, &id1, &pif1);
          if(SUCCEEDED(hr)) {
	        pif1->lpVtbl->GetValue(pif1,&result);
	        pif1->lpVtbl->SetValue(pif1,19);
	        pif1->lpVtbl->GetValue(pif1,&result);
	        pif1->lpVtbl->Release(pif1);
		    printf("CREATED IObject, value: %i (should be 19)\n",result);
          } else printf("**FAILED To Create IObject\n");
        } else printf("**FAILED To Get IID\n");

      } else printf("**FAILED To Get PROGID\n");
    }
#endif
	//DON'T FORGET
    CoUninitialize();

	return 0;
}

// That's all folks !

